@@ -30,7 +30,7 @@ testthat::test_file("tests/manual/test-code/test-Fease_Functions.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-Import_Data.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-MPs.R") # OK
+testthat::test_file("tests/manual/test-code/test-MPs.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-MSE_functions.R") # ok
 
@@ -40,13 +40,13 @@ testthat::test_file("tests/manual/test-code/test-OM_functions.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-OM_init_doc.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-OM_Plotting.R") # ok
+testthat::test_file("tests/manual/test-code/test-OM_Plotting.R") # fix errors
 
-testthat::test_file("tests/manual/test-code/test-PMobjects.R") # Ok
+testthat::test_file("tests/manual/test-code/test-PMobjects.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-RealIndices.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-runMSE.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-slotDescription.R") # OK
+testthat::test_file("tests/manual/test-code/test-slotDescription.R") # ok
 
